@@ -22,11 +22,11 @@ class MyTupBasic(NamedTuple):
     _a_str = {'choices': ['hello', 'bonjour', 'hola']}  # will overwrite the a_str argument choices
     b_list_int: List[int]
     b_set_str: Set[str]
-    c_optional_float: Optional[float]
     d_tuple_3: Tuple[int, float, bool]
     d_tuple_2: Tuple[str, int]
     e_dict_str_int: Dict[str, int]
     e_dict_int_bool: Dict[int, bool]
+    c_optional_float: Optional[float] = None
     with_default: int = 10
 
 
@@ -45,15 +45,15 @@ my_tup_basic = MyTupBasic(
 
 
 def test_basic_parse_to_arg():
-    with pytest.raises(TypeError, match="missing 11 required keyword arguments"):
+    with pytest.raises(TypeError, match="missing 10 required keyword arguments"):
         MyTupBasic()
-    arg_cmd = '--a_int 32 --a_float 0.3 --a_bool True --a_str hello --b_list_int 1 2 3 --b_set_str set1 set2 --c_optional_float None ' + \
-              '--d_tuple_3 10 0.5 False --d_tuple_2 tuple 12 --e_dict_str_int size:32 area:90 --e_dict_int_bool 10:True 20:False 30:True --with_default 10'
+    arg_cmd = 'MyTupBasic+ --a_int 32 --a_float 0.3 --a_bool True --a_str hello --b_list_int 1 2 3 --b_set_str set1 set2 ' + \
+              '--d_tuple_3 10 0.5 False --d_tuple_2 tuple 12 --e_dict_str_int size:32 area:90 --e_dict_int_bool 10:True 20:False 30:True --with_default 10 MyTupBasic-'
     parsed_arg = MyTupBasic(arg_cmd.split())
     assert parsed_arg == my_tup_basic
     parsed_arg_from_factory: MyTupBasic = MyTupBasic.__from_argv__(arg_cmd.split())
     assert parsed_arg == parsed_arg_from_factory
-    serialized_cmd_line = my_tup_basic.__to_argv__()
+    serialized_cmd_line = my_tup_basic.__to_argv__(separator=None)
     assert set(serialized_cmd_line) == set(arg_cmd.split())
     my_parser = MyTupBasic.__arg_suite__._parser
     assert my_parser._option_string_actions['--a_int'].help == '(int, required)  a is int'
@@ -70,12 +70,25 @@ def test_parse_error():
         pytest.raises(SystemExit, MyTupBasic, [])
 
 
+def test_optional():
+    @arg_suite
+    class MyTup(NamedTuple):
+        ints: Optional[List[int]] = None
+
+    pytest.raises(SystemExit, MyTup, ['--ints', 'None'])
+    # assert MyTup(['--ints', 'None']).ints is None
+    assert MyTup(['--ints', '1', '2']).ints == [1, 2]
+    assert MyTup(['--ints']).ints == []
+    pytest.raises(SmartArgError, arg_suite, NamedTuple('InvalidOptional', no=Optional[int]))
+
+
 def test_post_process():
     @arg_suite
     class MyTup(NamedTuple):
-        a_int: int = LateInit  # if a_int is not in the argument, post_process will initialize it
+        a_int: Optional[int] = LateInit  # if a_int is not in the argument, post_process will initialize it
 
     pytest.raises(SmartArgError, MyTup)
+    pytest.raises(SmartArgError, MyTup, a_int='not a int')
     pytest.raises(SmartArgError, MyTup, [])
     MyTup.__late_init__ = lambda s: None
     pytest.raises(SmartArgError, MyTup, [])
@@ -195,24 +208,37 @@ def test_unsupported_types():
 
 
 def test_nested():
+    @arg_suite
+    class MyTup(NamedTuple):
+        def __late_init__(self):
+            return self._replace(b_int=10) if self.b_int is LateInit else self
+        b_int: int = LateInit  # if a_int is not in the argument, __late_init__ will initialize it
+
     @arg_suite
     class Nested(NamedTuple):
         a_int: int
         nested: MyTupBasic = my_tup_basic  # nested
         another_int: int = 0
+        another_nested: MyTup = MyTup()
 
-    # Nested.__arg_suite__._parser.print_help()
     nested = Nested(a_int=0)
+    assert nested.another_nested.b_int == 10
     argv = nested.__to_argv__()
     assert Nested(argv) == nested
     assert Nested(argv[0:2]) == nested
     pytest.raises(SmartArgError, Nested, ['--a_int', '0', '--nested', 'raise'])
 
+    class NotDecoratedWithPost(NamedTuple):
+        def __validate__(self): pass
+
+    pytest.raises(SmartArgError, arg_suite, NamedTuple('Nested', nested=NotDecoratedWithPost))
+    arg_suite(NamedTuple('Nested', nested=NamedTuple('NotDecorated')))  # should not raise
+
 
 def test_cli_execution():
     demo = f'{sys.executable if sys.executable else "python"} {os.path.join(os.path.dirname(__file__), "smart_arg_demo.py")}'
     args = '--nn 200 300 --a_tuple s 5 --encoder fastText --h_param y:1 n:0 --nested.n_nested.n_' \
-           'name "nested name" --n None --embedding_dim 100 --lr 0.001'
+           'name "nested name" --embedding_dim 100 --lr 0.001'
     arg_line = demo + ' ' + args
     kwargs = {'stdout': subprocess.PIPE, 'stderr': subprocess.PIPE, 'shell': True}
     completed_process = subprocess.run(arg_line, **kwargs)
